SQLite — SQLAlchemy 0.6 Documentation

# SQLAlchemy 0.6 Documentation

Search:
Release: 0.6.9 | Release Date: May 5, 2012

# SQLite

Support for the SQLite database.

For information on connecting using a specific driver, see the documentation section regarding that driver.

## Date and Time Types

SQLite does not have built-in DATE, TIME, or DATETIME types, and pysqlite does not provide out of the box functionality for translating values between Python datetime objects and a SQLite-supported format. SQLAlchemy’s own `DateTime` and related types provide date formatting and parsing functionality when SQlite is used. The implementation classes are `DATETIME`, `DATE` and `TIME`. These types represent dates and times as ISO formatted strings, which also nicely support ordering. There’s no reliance on typical “libc” internals for these functions so historical dates are fully supported.

## Auto Incrementing Behavior

Background on SQLite’s autoincrement is at: http://sqlite.org/autoinc.html

Two things to note:

- The AUTOINCREMENT keyword is not required for SQLite tables to generate primary key values automatically. AUTOINCREMENT only means that the algorithm used to generate ROWID values should be slightly different.
- SQLite does not generate primary key (i.e. ROWID) values, even for one column, if the table has a composite (i.e. multi-column) primary key. This is regardless of the AUTOINCREMENT keyword being present or not.

To specifically render the AUTOINCREMENT keyword on the primary key column when rendering DDL, add the flag `sqlite_autoincrement=True` to the Table construct:

    Table('sometable', metadata,
            Column('id', Integer, primary_key=True),
            sqlite_autoincrement=True)

## Transaction Isolation Level

`create_engine()` accepts an `isolation_level` parameter which results in the command `PRAGMA read_uncommitted <level>` being invoked for every new connection. Valid values for this parameter are `SERIALIZABLE` and `READ UNCOMMITTED` corresponding to a value of 0 and 1, respectively.

## SQLite Data Types

As with all SQLAlchemy dialects, all UPPERCASE types that are known to be valid with SQLite are importable from the top level dialect, whether they originate from `sqlalchemy.types` or from the local dialect:

    from sqlalchemy.dialects.sqlite import \
                BLOB, BOOLEAN, CHAR, DATE, DATETIME, DECIMAL, FLOAT, \
                INTEGER, NUMERIC, SMALLINT, TEXT, TIME, TIMESTAMP, \
                VARCHAR

class `sqlalchemy.dialects.sqlite.` `DATETIME`(storage_format=None, regexp=None, **kwargs)

  Represent a Python datetime object in SQLite using a string.

  The default string storage format is:

      "%04d-%02d-%02d %02d:%02d:%02d.%06d" % (value.year,
                              value.month, value.day,
                              value.hour, value.minute,
                              value.second, value.microsecond)

  e.g.:

      2011-03-15 12:05:57.10558

  The storage format can be customized to some degree using the `storage_format` and `regexp` parameters, such as:

      import re
      from sqlalchemy.dialects.sqlite import DATETIME

      dt = DATETIME(
              storage_format="%04d/%02d/%02d %02d-%02d-%02d-%06d",
              regexp=re.compile("(\d+)/(\d+)/(\d+) (\d+)-(\d+)-(\d+)(?:-(\d+))?")
          )

  Parameters: | storage_format – format string which will be appled to the tuple `(value.year, value.month, value.day, value.hour, value.minute, value.second, value.microsecond)`, given a Python datetime.datetime() object. regexp – regular expression which will be applied to incoming result rows. The resulting match object is appled to the Python datetime() constructor via `*map(int, match_obj.groups(0))`.

class `sqlalchemy.dialects.sqlite.` `DATE`(storage_format=None, regexp=None, **kwargs)

  Represent a Python date object in SQLite using a string.

  The default string storage format is:

      "%04d-%02d-%02d" % (value.year, value.month, value.day)

  e.g.:

      2011-03-15

  The storage format can be customized to some degree using the `storage_format` and `regexp` parameters, such as:

      import re
      from sqlalchemy.dialects.sqlite import DATE

      d = DATE(
              storage_format="%02d/%02d/%02d",
              regexp=re.compile("(\d+)/(\d+)/(\d+)")
          )

  Parameters: | storage_format – format string which will be appled to the tuple `(value.year, value.month, value.day)`, given a Python datetime.date() object. regexp – regular expression which will be applied to incoming result rows. The resulting match object is appled to the Python date() constructor via `*map(int, match_obj.groups(0))`.

class `sqlalchemy.dialects.sqlite.` `TIME`(storage_format=None, regexp=None, **kwargs)

  Represent a Python time object in SQLite using a string.

  The default string storage format is:

      "%02d:%02d:%02d.%06d" % (value.hour, value.minute,
                              value.second,
                               value.microsecond)

  e.g.:

      12:05:57.10558

  The storage format can be customized to some degree using the `storage_format` and `regexp` parameters, such as:

      import re
      from sqlalchemy.dialects.sqlite import TIME

      t = TIME(
              storage_format="%02d-%02d-%02d-%06d",
              regexp=re.compile("(\d+)-(\d+)-(\d+)-(?:-(\d+))?")
          )

  Parameters: | storage_format – format string which will be appled to the tuple `(value.hour, value.minute, value.second, value.microsecond)`, given a Python datetime.time() object. regexp – regular expression which will be applied to incoming result rows. The resulting match object is appled to the Python time() constructor via `*map(int, match_obj.groups(0))`.

## Pysqlite

Support for the SQLite database via pysqlite.

Note that pysqlite is the same driver as the `sqlite3` module included with the Python distribution.

### Driver

When using Python 2.5 and above, the built in `sqlite3` driver is already installed and no additional installation is needed. Otherwise, the `pysqlite2` driver needs to be present. This is the same driver as `sqlite3`, just with a different name.

The `pysqlite2` driver will be loaded first, and if not found, `sqlite3` is loaded. This allows an explicitly installed pysqlite driver to take precedence over the built in one. As with all dialects, a specific DBAPI module may be provided to `create_engine()` to control this explicitly:

    from sqlite3 import dbapi2 as sqlite
    e = create_engine('sqlite+pysqlite:///file.db', module=sqlite)

Full documentation on pysqlite is available at: http://www.initd.org/pub/software/pysqlite/doc/usage-guide.html

### Connect Strings

The file specification for the SQLite database is taken as the “database” portion of the URL. Note that the format of a url is:

    driver://user:pass@host/database

This means that the actual filename to be used starts with the characters to the right of the third slash. So connecting to a relative filepath looks like:

    # relative path
    e = create_engine('sqlite:///path/to/database.db')

An absolute path, which is denoted by starting with a slash, means you need four slashes:

    # absolute path
    e = create_engine('sqlite:////path/to/database.db')

To use a Windows path, regular drive specifications and backslashes can be used. Double backslashes are probably needed:

    # absolute path on Windows
    e = create_engine('sqlite:///C:\\path\\to\\database.db')

The sqlite `:memory:` identifier is the default if no filepath is present. Specify `sqlite://` and nothing else:

    # in-memory database
    e = create_engine('sqlite://')

### Compatibility with sqlite3 “native” date and datetime types

The pysqlite driver includes the sqlite3.PARSE_DECLTYPES and sqlite3.PARSE_COLNAMES options, which have the effect of any column or expression explicitly cast as “date” or “timestamp” will be converted to a Python date or datetime object. The date and datetime types provided with the pysqlite dialect are not currently compatible with these options, since they render the ISO date/datetime including microseconds, which pysqlite’s driver does not. Additionally, SQLAlchemy does not at this time automatically render the “cast” syntax required for the freestanding functions “current_timestamp” and “current_date” to return datetime/date types natively. Unfortunately, pysqlite does not provide the standard DBAPI types in `cursor.description`, leaving SQLAlchemy with no way to detect these types on the fly without expensive per-row type checks.

Keeping in mind that pysqlite’s parsing option is not recommended, nor should be necessary, for use with SQLAlchemy, usage of PARSE_DECLTYPES can be forced if one configures “native_datetime=True” on create_engine():

    engine = create_engine('sqlite://',
                    connect_args={'detect_types': sqlite3.PARSE_DECLTYPES|sqlite3.PARSE_COLNAMES},
                    native_datetime=True
                    )

With this flag enabled, the DATE and TIMESTAMP types (but note - not the DATETIME or TIME types...confused yet ?) will not perform any bind parameter or result processing. Execution of “func.current_date()” will return a string. “func.current_timestamp()” is registered as returning a DATETIME type in SQLAlchemy, so this function still receives SQLAlchemy-level result processing.

### Threading Behavior

Pysqlite connections do not support being moved between threads, unless the `check_same_thread` Pysqlite flag is set to `False`. In addition, when using an in-memory SQLite database, the full database exists only within the scope of a single connection. It is reported that an in-memory database does not support being shared between threads regardless of the `check_same_thread` flag - which means that a multithreaded application cannot share data from a `:memory:` database across threads unless access to the connection is limited to a single worker thread which communicates through a queueing mechanism to concurrent threads.

To provide a default which accomodates SQLite’s default threading capabilities somewhat reasonably, the SQLite dialect will specify that the `SingletonThreadPool` be used by default. This pool maintains a single SQLite connection per thread that is held open up to a count of five concurrent threads. When more than five threads are used, a cleanup mechanism will dispose of excess unused connections.

Two optional pool implementations that may be appropriate for particular SQLite usage scenarios:

> - the `sqlalchemy.pool.StaticPool` might be appropriate for a multithreaded application using an in-memory database, assuming the threading issues inherent in pysqlite are somehow accomodated for. This pool holds persistently onto a single connection which is never closed, and is returned for all requests.
> - the `sqlalchemy.pool.NullPool` might be appropriate for an application that makes use of a file-based sqlite database. This pool disables any actual “pooling” behavior, and simply opens and closes real connections corresonding to the `connect()` and `close()` methods. SQLite can “connect” to a particular file with very high efficiency, so this option may actually perform better without the extra overhead of `SingletonThreadPool`. NullPool will of course render a `:memory:` connection useless since the database would be lost as soon as the connection is “returned” to the pool.

### Unicode

In contrast to SQLAlchemy’s active handling of date and time types for pysqlite, pysqlite’s default behavior regarding Unicode is that all strings are returned as Python unicode objects in all cases. So even if the `Unicode` type is not used, you will still always receive unicode data back from a result set. It is strongly recommended that you do use the `Unicode` type to represent strings, since it will raise a warning if a non-unicode Python string is passed from the user application. Mixing the usage of non-unicode objects with returned unicode objects can quickly create confusion, particularly when using the ORM as internal data is not always represented by an actual database result string.